import os
from os.path import basename, join, splitext
from itertools import groupby
import re
from glob import glob
from typing import List

from ..util import replace_ext, gdal


def stack_bands_step(source_dir: os.PathLike, target_dir: os.PathLike, group_by: str=None, sort_by: str=None, order: List[str]=None):
    """ Stack bands of the individual images
    """
    filenames = glob(join(source_dir, '*'), recursive=True)
    # check if we have a group_by regex. If yes, use the first
    # re-group to group by.
    # Fallback is basename of file as groupname
    if group_by:
        re_group_by = re.compile(group_by)
        groups = {
            k: list(v)
            for k, v in groupby(filenames, key=lambda v: re_group_by.match(v).group(1))
        }
    else:
        groups = {basename(filenames[0]): filenames}

    for groupname, group in groups.items():
        # check if a sort_by is specified. if yes, use the sort_by regex group
        # and optionally a ordered list to order the filenames
        if sort_by:
            re_sort_by = re.compile(sort_by)

            if order:
                group = [
                    v for v in group
                    if re_sort_by.match(v)
                    and re_sort_by.match(v).group(1) in order
                ]

                group = sorted(
                    group,
                    key=lambda v: order.index(re_sort_by.match(v).group(1))
                )
            else:
                group = sorted(
                    group,
                    key=lambda v: re_sort_by.match(v).group(1)
                )

        # build a VRT to stack bands for each group
        vrt_filename = replace_ext(join(target_dir, groupname), '.vrt')
        gdal.BuildVRT(vrt_filename, group, separate=True)
