import os
from os.path import splitext
from contextlib import contextmanager
from tempfile import TemporaryDirectory, mkdtemp

try:
    from osgeo import gdal
except ImportError:
    import gdal

gdal.UseExceptions()

try:
    from osgeo import osr
except ImportError:
    import osr

osr.UseExceptions()


def replace_ext(filename: os.PathLike, new_ext: str, force_dot: bool=True) -> os.PathLike:
    return splitext(filename)[0] + ('' if new_ext.startswith('.') or not force_dot else '.') + new_ext


@contextmanager
def workdir(config: dict, use_dir: os.PathLike=None):
    prefix = config.get('prefix', 'preprocess_')
    workdir = config.get('workdir')
    if use_dir:
        os.chdir(use_dir)
        yield use_dir
    elif config.get('keep_temp'):
        dirname = mkdtemp(prefix=prefix, dir=workdir)
        os.chdir(dirname)
        yield dirname
    else:
        with TemporaryDirectory(prefix=prefix, dir=workdir) as dirname:
            os.chdir(dirname)
            yield dirname


def pairwise(col):
    iterator = iter(col)
    while True:
        try:
            yield (next(iterator), next(iterator))
        except StopIteration:
            break
